--Resound music rework for Vendetta Online

local dbg = true

local resound = {}

local cp = function(input, case)
	input = "[Resound] " .. tostring(input)
	lib.log_error(input, tonumber(case) or 1, "Resound", "3.0.0")
end

--babel support

local bstr = function(id, val)
	return val
end

local babel, ref_id, update_class
local do_babel = function()
	if lib.is_ready("babel", "0") then
		cp("Applying language table to Resound")
		babel = lib.get_class("babel")
		ref_id = babel.register("plugins/Resound/lang/", {'en', 'es', 'fr', 'pt'})
		
		bstr = function(id, val)
			return babel.fetch(ref_id, id, val)
		end
		
		resound.add_translation = function(path, lang_code)
			babel.add_new_lang(ref_id, path, lang_code)
		end
		
		update_class()
	end
end










local gkrs = gkini.ReadString

local config = {
	enable_resound = gkrs("Resound", "enable_resound", "YES"),
	vol = tonumber(Game.GetCVar("snd_musicvolume")),
	tick_time = gkrs("Resound", "tick_time", "10"),
		--period between sector scans for tag changes
	seek_songs = gkrs("Resound", "seek_songs", "50"),
		--how many songs to check for tag validity
	seek_style = gkrs("Resound", "seek_style", "iter"),
		--iter: Iterative timer, will not hang the game
		--hard: Hard loop, find tracks faster but may hang on LARGE playlists
	adv_tags = gkrs("Resound", "adv_tags", "YES"), --do advanced tags
	do_notifs = gkrs("Resound", "do_notifs", "NO"), --LME notif on new song
	do_chat = gkrs("Resound", "do_chat", "YES"), --say in chat on new song
}



update_class = function()
	local class = {
		CCD1 = true,
		open = nil,
		config = nil,
		smart_config = {
			title = bstr(1, "Resound"),
			cb = function(id, val)
				if config[id] then
					config[id] = val
					gkini.WriteString("Resound", id, val)
				end
			end,
			[1] = "notice",
			notice = {
				type = "text",
				display = "Resound is a release candidate! Settings and features may change!",
			},
			[2] = "spacer",
			spacer = {
				type = "spacer",
			},
			[3] = "tick_time",
			tick_time = {
				type = "input",
				display = bstr(2, "Scanning function interval"),
				default = config.tick_time,
			},
			[4] = "seek_songs",
			seek_songs = {
				type = "input",
				display = bstr(3, "Number of songs to try (legacy seek only)"),
				default = config.seek_songs,
			},
			[5] = "seek_style",
			seek_style = {
				type = "dropdown",
				display = bstr(8, "Track Seek Style"),
				default = config.adv_tags,
				[1] = bstr(9, "Iterative (safe)"),
				[2] = bstr(10, "Hard Loop (faster)"),
			},
			[6] = "adv_tags",
			adv_tags = {
				type = "toggle",
				display = bstr(4, "Enable optional tags"),
				[1] = config.adv_tags,
			},
			[7] = "do_notifs",
			do_notifs = {
				type = "toggle",
				display = bstr(5, "Show LME notification on track change"),
				[1] = config.do_notifs,
			},
			[8] = "do_chat",
			do_chat = {
				type = "toggle",
				display = bstr(6, "Show chat messages on track change"),
				[1] = config.do_chat,
			},
		},
		manifest = {
			"plugins/Resound/main.lua",
			"plugins/Resound/registration.ini",
			"plugins/Resound/resound3.lua",
			"plugins/Resound/silent.ogg",
			"plugins/Resound/default.ini",
			"plugins/Resound/extra_tags.lua",
			"plugins/Resound/lang/en.ini",
			"plugins/Resound/lang/es.ini",
			"plugins/Resound/lang/fr.ini",
			"plugins/Resound/lang/pt.ini",
		},
		commands = {''},
	}
	
	for k, v in pairs(class) do
		resound[k] = v
	end
	
	lib.set_class("resound", "0", resound)
end







local system_faction = {
	[1] = "serco", --sol II
	[2] = "serco",
	[3] = "serco",
	[4] = "grey", --??? Deneb
	[5] = "itani",
	[6] = "itani",
	[7] = "itani",
	[8] = "itani",
	[9] = "itani", --itan
	[10] = "itani",
	[11] = "itani",
	[12] = "itani",
	[13] = "itani", --jallik
	[14] = "grey", --edras, and the last sane straight line
	[15] = "uit", --verasi
	[16] = "grey", --pelatus
	[17] = "grey", --bractus
	[18] = "uit", --nyrius
	[19] = "uit", --dau
	[20] = "grey", --sedina
	[21] = "uit", --azek
	[22] = "grey", --odia
	[23] = "grey", --latos
	[24] = "uit", --arta cel
	[25] = "grey", --ukari
	[26] = "grey", --helios
	[27] = "serco", --initros, and back to sanity
	[28] = "serco", --pyronis
	[29] = "serco", --Rhamus
	[30] = "serco", --dantia; inner above, outer below
	[31] = "grey", --Devlopia
	[32] = "grey", --doesn't exist. 
	[33] = "grey", --Infernus/Unknown System
}

local function get_system_alignment()
	return system_faction[GetCurrentSystemid()] or "grey"
end







local engine = {
	library = {
		--[[
		album_name = {
			enabled = "YES/NO",
			song_name = {
				numeric_index,
				name,
				file,
				artist,
				length,
				tags,
				custom,
			},
		},
		]]--
		resound = {
			silence = {
				index = 1,
				name = "silence",
				file = "plugins/Resound/silent.ogg",
				album = "resound",
				artist = "no artist",
				length = 0,
				tags = "=never",
			},
		},		
	},
	playlist = {},
	last_stop_position = 1,
	data = {
		playing = {
			album = "Vendetta Online Soundtrack",
			name = "Vendetta Online Menu Theme",
			length = "-1",
			artist = "Guild Software",
			tags = "=never",
			file = "music/menu.ogg",
			
			song_start = os.time(),
		},
		history = {
			[1] = {
				"Vendetta Online Soundtrack",
				"Vendetta Online Menu Theme",
			}, --always triggered by game first start
		},
	},
	first_start = true,
	is_docked = false,
	is_playing = false,
	ignore_stop = false,
	is_sector_transition = false,
	is_track_held = false, --sector_changed event occured
	is_seeking = false, --delay :stop() till after :seek() completes
	lock_current_track = false, --prevent track seeking
	reset_tag_check = false, --true when resound needs to reseek a song
	
	sector_tags = {},
	forced_tags = {},
	
	--tag funcs
	
	toggle_tag = function(self, tag, status)
		if type(tag) ~= "string" then
			return false
		end
		if type(status) ~= "boolean" then
			status = nil
		end
		
		if status == nil then
			status = not (self.sector_tags[tag] or false)
		end
		
		self.sector_tags[tag] = status
	end,
	
	require_tag = function(self, tag)
		tag = tostring(tag)
		if self.forced_tags[tag] then
			return
		end
		self.forced_tags[tag] = true
		self:toggle_tag(tag, true)
		self.reset_tag_check = true
	end,
	
	clear_tags = function(self)
		self.sector_tags = {}
		self.forced_tags = {}
	end,
	
	get_tag = function(self, tag)
		if type(tag) ~= "string" then
			return false
		end
		return self.forced_tags[tag] or self.sector_tags[tag] or false
	end,
	
	try_tags = function(self, taglist)
		-- split tags with prefixes
		if config.enable_resound == "NO" then
			return false
		end
		local tagtable = {}
		for tag in string.gmatch(tostring(taglist), "[%=%+%-]?%w+") do
			local prefix = string.sub(tag, 1, 1)
			local tagValue = string.sub(tag, 2)
			tagtable[tagValue] = prefix
		end
		
		--check if required tags exist; throw out this track if invalid
		local abort = false
		for required in pairs(self.forced_tags) do
			if not tagtable[required] then
				abort = true
				break
			end
		end
		
		if abort then
			return false
		end
		
		--check if all existing prefixes are met
		local valid = false
		for tag, state in pairs(tagtable) do
			if state == "-" and self.sector_tags[tag] then
				abort = true
				break
			elseif state == "+" and self.sector_tags[tag] then
				valid = true
			elseif state == "=" then
				if not self.sector_tags[tag] then
					abort = true
					break
				else
					valid = true
				end
			end
		end
		
		if abort or not valid then
			-- if abort, toss out because something does/n't exist when it should/n't
			-- if not valid, toss out because no += tags required by this song were present
			return false
		else
			return true
		end
		
	end,
	
	--song grabbers
	
	get_album_list = function(self)
		local tab = {}
		for k, _ in pairs(self.library) do
			table.insert(tab, k)
		end
		return tab
	end,
	
	get_song_list = function(self, album_name)
		if not self.library[album_name] then
			return false
		end
		local tab = {}
		for k, v in pairs(self.library[album_name]) do
			if type(v) == "table" then
				table.insert(tab, k)
			end
		end
		return tab
	end,
	
	get_song_details = function(self, album_name, song_name)
		if not (self.library[album_name] and self.library[album_name][song_name]) then
			return false
		end
		
		return self.library[album_name][song_name]
	end,
	
	--creators
	
	new_album = function(self, ini_pointer)
		
		if not gksys.IsExist(ini_pointer) then
			return false
		end
		
		local gkstr = function(head, key, val)
			return gkini.ReadString2(head, key, val, ini_pointer)
		end
		
		local album_name = gkstr("resound", "album", "null")
		if new_album == "null" then
			return false
		end
		
		cp("Adding new album: " .. album_name)
		
		local album = {}
		
		for track_index = 1, tonumber(gkstr("resound", "tracks", "1")) do
			local tid = tostring(track_index) --save typing
			local track_name = gkstr(tid, "title", "null")
			album[track_name] = {
				index = tid,
				name = track_name,
				file = gkstr(tid, "file", "null"),
				artist = gkstr(tid, "artist", "unknown"),
				album = album_name,
				length = gkstr(tid, "length", "-1"),
				tags = gkstr(tid, "tags", "=never"),
				vol = gkstr(tid, "vol", "100"),
			}
			cp("	indexed track " .. tid .. ": " .. track_name)
		end
		
		album.enabled = gkini.ReadString("Resound", album_name, "YES")
		
		if gkstr("resound", "custom", "NO") ~= "NO" then
			lib.resolve_file(gkstr("resound", "custom", "NO"))
		end
		
		self.library[album_name] = album
		
		self:generate_playlist()
	end,
	
	new_song = function(self, album_name, song_table)
		
		if type(song_table) ~= "table" then
			return false
		end
		
		if not self.library[album_name] then
			return false
		end
		
		local default = {
			name = "null",
			file = "plugins/Resound/silent.ogg",
			artist = "Unknown",
			album = album_name,
			length = "-1",
			tags = "=never",
			vol = "100",
			custom = nil,
		}
		
		for k, v in pairs(song_table) do
			default[k] = v
		end
		
		
		
		self.library[album_name][default.name] = default
	end,
	
	generate_playlist = function(self)
		if config.enable_resound == "NO" then
			return false
		end
		
		--reads through the library and creates a list of songs that are enabled. Does not limit songs to sector-valid ones; use :create_tracklist() for that.
		cp("Creating new playlist")
		self.playlist = {}
		
		for album, tracklist in pairs(self.library) do
			if tracklist.enabled ~= "YES" then
				cp("Skipping album " .. album .. ": not enabled!")
			else
				
				cp("Adding album " .. album .. " to playlist")
				
				for track, data in pairs(tracklist) do
					if type(data) == "table" then
						table.insert(self.playlist, {album, track})
					else
						cp(tostring(track) .. " is not a table for playback")
					end
				end
			end
		end
		
		self:shuffle_playlist()
	end,
	
	shuffle_playlist = function(self)
		local t = self.playlist
		--Fisher-Yates shuffle algorithm
		for i = #t, 2, -1 do
			local j = math.random(i)
			t[i], t[j] = t[j], t[i]
		end
	end,
	
	-- tag determination
	
	on_tick_funcs = {
		[1] = function(self)
			--check sector's "danger"
			local skip = false
			ForEachPlayer(
				function(id)
					if skip then
						return
					end
					local val = GetFriendlyStatus(id)
					if val < 3 then
						if not self:get_tag("danger") then
							self:toggle_tag("danger", true)
							self.reset_tag_check = true
							skip = true
							console_print("danger tag tripped on a user")
						end
					end
				end
			)
		end,
	},
	
	new_scanning_tag = function(self, func)
		cp("New scanning function!")
		table.insert(self.on_tick_funcs, func)
	end,
	
	on_entry_funcs = {
		[1] = function(self)
			--System Faction Alignment
			self:toggle_tag(get_system_alignment(), true)
		end,
		[2] = function(self)
			--Station check
			self:toggle_tag("station", (GetSectorMonitoredStatus() or 1) > 2 and true or false)
		end,
		[3] = function(self)
			--Ambient sector check
			self:toggle_tag("ambient", (GetSectorMonitoredStatus() or 1) < 3 and true or false)
		end,
		
	},
	
	new_entry_tag = function(self, func)
		cp("New entry function!")
		table.insert(self.on_entry_funcs, func)
	end,
	
	-- interface event
	
	interfaces = {},
	
	register_resound_update_callback = function(self, func)
		cp("New callback for resound updates!")
		table.insert(self.interfaces, func)
	end,
	
	interface_update_trigger = function(self)
		--run to notify resound dependents that a change occured
		for k, v in ipairs(self.interfaces) do
			if type(v) == "function" then
				v(self)
			end
		end
	end,
	
	-- playback funcs
	
	last_playing_time = os.time(),
	set_playing = function(self, album, song)
		if config.enable_resound == "NO" then
			return false
		end
		
		if os.time() - self.last_playing_time < 1 and self.is_playing then
			console_print("[Resound] Skipped play event; triggered too soon after last track change!")
			return false
		end
		
		self.last_playing_time = os.time()
		
		local song_data = self.library[album][song]
		
		cp("Playing track: " .. song_data.name)
		
		table.insert(self.data.history, {album, song})
		self.data.playing = song_data
		self.data.playing.start_time = os.time()
		
		gksound.GKPlayMusic(song_data.file, true, true, nil, (tonumber(song_data.vol or 100)/100))
		
		self.is_playing = true
		self.reset_tag_check = false
		
		self:interface_update_trigger()
	end,
	
	Play = function(self, album, song)
		if config.enable_resound == "NO" then
			return false
		end
		--find a particular song and play it; calls seek otherwise
		
		if type(album) == "number" then
			--playing by playlist index
			song = self.playlist[album]
			album, song = song[1], song[2]
		end
		
		if self.library[album] and self.library[album][song] then
			local song_data = self.library[album][song]
			
			self:set_playing(album, song)
		else
			--song doesn't exist, triggering seek instead
			console_print("Seek triggered, attempted :play() but valid song not provided")
			self:Seek()
		end
	end,
	
	Stop = function(self)
		if config.enable_resound == "NO" then
			return false
		end
		
		--playing a silent track will fade out the music
		--using the GKStopMusic function hard-cuts the music
		local stop_Timer = Timer()
		local stop_processor
		stop_processor = function()
			if self.is_seeking then
				stop_Timer:SetTimeout(1, stop_processor)
			else
				if not self.ignore_stop then
					gksound.GKPlayMusic("plugins/Resound/silent.ogg", true)
					self.is_playing = false
					self.reset_tag_check = false
					self.ignore_stop = false
				end
				stop_Timer:Kill()
				console_print("Halted!")
			end
		end
		stop_processor()
	end,
	
	Seek = function(self)
		if self.is_seeking then
			console_print("Skipped a Seek; already set to seek!")
		end
		self.is_seeking = true
		cp("Seek called, checking against tags " .. spickle(self.sector_tags))
		if config.seek_style == "iter" then
			self:Seek1(self)
		elseif config.seek_style == "hard" then
			self:Seek2(self)
		else
			cp("seek method not recognised - incorrect version maybe? " .. tostring(config.seek_style))
		end
	end,
	
	Seek1 = function(self)
		if config.enable_resound == "NO" then
			return false
		end
		
		--finds the next valid track
			local timestart = gkmisc.GetGameTime()
		if self.lock_current_track then
			return
		end
		
		local current_index = self.last_stop_position
		local counter = 0
		local seekTimer = Timer()
		local once = true
		local function seek_iterator(self)
			if once then
				once = false
				cp("iterator hit idle")
			end
			current_index = current_index + 1
			counter = counter + 1
			if counter > self.last_stop_position + #self.playlist then
				cp("reached end of playlist!")
				return false
			elseif current_index > #self.playlist then
				current_index = 1 -- playlist looping
				self:shuffle_playlist()
			end
			
			local song_entry = self.playlist[current_index]
			if type(song_entry) == "table" then
				local song_data = self.library[song_entry[1]][song_entry[2]]
				local valid_for_play = self:try_tags(song_data.tags)
				
				if valid_for_play and type(song_data.custom) == "function" then
					valid_for_play = valid_for_play and song_data.custom(self)
				end
				
				if valid_for_play then
					self:set_playing(song_entry[1], song_entry[2])
					self.last_stop_position = current_index
					seekTimer:Kill()
					self.is_seeking = false
					
						local timestop = gkmisc.GetGameTime()
						cp("seek took " .. tostring(timestop - timestart) .. "ms")
					return true
				end
			end
			seekTimer:SetTimeout(1, function() seek_iterator(self) end)
		end
		seekTimer:SetTimeout(1, function() seek_iterator(self) end)
		console_print("seek iterator launched")
		
		
	end,
	
	Seek2 = function(self)
		if config.enable_resound == "NO" then
			return false
		end
		
		--finds the next valid track
			local timestart = gkmisc.GetGameTime()
		if self.lock_current_track then
			return
		end
		
		local current_index = self.last_stop_position
		local counter = 0
		while true do
			current_index = current_index + 1
			counter = counter + 1
			if counter > self.last_stop_position + #self.playlist then
				cp("reached end of playlist!")
				break -- made a whole cycle and found no valid tracks!
			elseif current_index > #self.playlist then
				current_index = 1 -- playlist looping
				self:shuffle_playlist()
			elseif counter > tonumber(config.seek_songs) then
				cp("reached song seek limit! (legacy seek)")
				break -- reached seek limit and found no valid tracks!
			end
			
			local song_entry = self.playlist[current_index]
			if type(song_entry) == "table" then
				local song_data = self.library[song_entry[1]][song_entry[2]]
				
				local valid_for_play = self:try_tags(song_data.tags)
				
				if valid_for_play and type(song_data.custom) == "function" then
					valid_for_play = valid_for_play and song_data.custom(self)
				end
				
				if valid_for_play then
					self:set_playing(song_entry[1], song_entry[2])
					self.last_stop_position = current_index
					self.is_seeking = false
					break
				end
			end
		end
		self.last_stop_position = current_index
		
		
			local timestop = gkmisc.GetGameTime()
			cp("seek took " .. tostring(timestop - timestart) .. "ms")
	end,
}
	

-- on-tick processor

local tick_timer = Timer()
local next_func = 1
local tick_processor
local entry_recent = false
tick_processor = function()
		if config.enable_resound == "NO" then
			return false
		end
		
	if engine.lock_current_track then
		tick_timer:SetTimeout(1000 * tonumber(config.tick_time), tick_processor)
		return
	end
	
	local nextrun = engine.on_tick_funcs[next_func]
	if type(nextrun) == "function" then
		nextrun(engine) --pcall this?
	end
	
	next_func = next_func + 1
	
	if next_func > #engine.on_tick_funcs then
		next_func = 1
	end
	
	if engine.reset_tag_check and not entry_recent then
		--this might cause doubled playback during sector entry - hence why the playback functions cannot instantly be called, and this checks entry recency
		console_print("reset tag check and not recent entry, attempting seek")
		engine:Seek()
	end
	
	tick_timer:SetTimeout((next_func == 1 and (1000 * tonumber(config.tick_time))) or 1, tick_processor)
end

-- on-entry processor
local entry_processor = function()
		if config.enable_resound == "NO" then
			return false
		end
		
	if engine.lock_current_track then
		return
	end
	
	engine.is_track_held = false
	
	for index, action in ipairs(engine.on_entry_funcs) do
		if type(action) == "function" then
			action(engine)
		end
	end
	entry_recent = true
	tick_processor()
	entry_recent = false
	
	
	console_print("entry_processor calling seek")
	engine:Seek()
	
	local taglist = ""
	for k, v in pairs(engine.sector_tags) do
		if v then
			taglist = taglist .. k .. " "
		end
	end
	
	cp("tags updated: " .. taglist)
end

engine.force_update = function(self)
	entry_processor()
end


-- events

--RegisterEvent(entry_processor, "PLAYER_ENTERED_GAME")

local conp = console_print

RegisterEvent(
	function()
		conp("Sector Loaded Event")
		conp(spickle{
			engine.first_start,
			engine.is_track_held,
			engine.is_playing,
			engine.is_docked,
		})
		if engine.is_track_held then
			return
		end
		for i, v in ipairs {
			engine.first_start,
			--engine.is_track_held,
			engine.is_playing,
		} do
			if v then
				if engine.is_seeking and i == 1 then
					conp("ignoring next stop, rule by sector load event")
					engine.ignore_stop = true
				end
				engine.first_start = false
				entry_processor()
				break
			end
		end
	end,
	"SECTOR_LOADED"
)

RegisterEvent(
	function()
		engine.is_docked = true
	end,
	"ENTERED_STATION"
)

RegisterEvent(
	--prevent soundtrack change during undock or first run
	--hello, me, why do I check first-run here?
	function()
	conp("Ship_spawn event")
	console_print( spickle {
		engine.first_start,
		engine.is_track_held,
		engine.is_playing,
		engine.is_docked,
	} )
		engine.is_docked = false
		engine.first_start = false
		if engine.is_track_held then
			entry_processor()
		end
	end,
	"SHIP_SPAWN_CINEMATIC_FINISHED"
)

RegisterEvent(
	function()
	conp("Sector changed Event")
	conp(spickle{
		engine.first_start,
		engine.is_track_held,
		engine.is_playing,
		engine.is_docked,
	})
		--if not engine.first_start then
			cp("Stopping track, sector is changing")
			engine:Stop()
			engine:clear_tags()
			if not engine.first_start then
				engine.is_track_held = true
			end
		--end
	end,
	"SECTOR_CHANGED"
)

RegisterEvent(function()
	
	conp("Player entered game Event")
	conp(spickle{
		engine.first_start,
		engine.is_track_held,
		engine.is_playing,
		engine.is_docked,
	})
	entry_processor()
	end, "PLAYER_ENTERED_GAME")



--obtain a reference copy of the rse
resound.get_rse = function() 
	return engine 
end

resound.new_album = function(instr)
	engine:new_album(tostring(instr))
end

resound.play = function(t)
	if type(t) == "table" and t[1] and t[2] then
		engine:Play(t[1], t[2])
	else
		engine:Seek()
	end
end

resound.play_random = function()
	local song_select = math.random(1, #engine.playlist)
	
	engine:Play(song_select)
end

resound.stop = function()
	engine:Stop()
end

resound.on_update = function(func)
	engine:register_resound_update_callback(func)
end

resound.get_playing = function()
	return engine.data.playing
end

resound.set_volume = function(new_volume)
	if not new_volume then
		new_volume = config.vol
	end
	
	config.vol = tonumber(new_volume)
	Game.SetCVar("snd_musicvolume", tostring(new_volume))
	return config.vol
end

resound.get_playlist = function()
	return engine.playlist
end

resound.get_albums = function()
	return engine:get_album_list()
end

resound.toggle_album_playback = function(album, flag)
	if not engine.library[album] then
		return "ERR"
	end
	
	if type(flag) == "boolean" then
		flag = (flag == true) and "YES" or "NO"
	end
	
	if flag == nil then
		flag = engine.library[album].enabled
	end
	
	if type(flag) ~= "string" then
		flag = "YES"
	end
	
	cp("Toggled album playback for " .. album .. " to " .. flag)
	
	engine.library[album].enabled = flag
	gkini.WriteString("Resound", album, flag)
	
	return flag
end

resound.toggle_resound = function(flag)
	if type(flag) == "boolean" then
		flag = (flag == true) and "YES" or "NO"
	end
	
	if flag == nil then
		flag = config.enable_resound
	end
	
	if type(flag) ~= "string" then
		flag = "YES"
	end
	
	config.enable_resound = flag
	gkini.WriteString("Resound", "enable_resound", flag)
	return flag
end

resound.get_settings = function()
	return config
end

resound.set_setting = function(id, flag)
	if not config[id or 0] then
		return false
	end
	
	if type(flag) ~= "string" then
		flag = config[id]
	else
		config[id] = flag
	end
	
	return flag
end

resound.get_songs = function(album)
	return engine:get_song_list(album)
end

resound.get_song_data = function(album, song)
	return engine:get_song_details(album, song)
end

resound.force_update = function()
	engine:generate_playlist()
	entry_processor()
end




lib.set_class("resound", "3.0.0", resound)

resound.new_album("plugins/Resound/default.ini")












local notif_handler = function(rse)
		if config.enable_resound == "NO" then
			return false
		end
		
	local song_data = rse.data.playing
	local time_readable = "0/?ms"
	
	if type(song_data) ~= "table" then
		--not ready
		return
	end
	
	if tonumber(song_data.length) > 0 then
		local song_time = math.floor(tonumber(song_data.length) % 1000)
		local seconds = song_time % 60
		song_time = math.floor(song_time / 60)
		local minutes = song_time % 60
		local hours = math.floor(song_time / 60)
		
		time_readable = string.format("%02d:%02d:%02d", hours, minutes, seconds)
	end
	
	if config.do_notifs == "YES" then
		lib.notify("RESOUND_NEW_TRACK", {
			title = "Resound",
			subtitle = bstr(7, "Now playing: "),
			line_1 = song_data.name,
			line_2 = song_data.artist,
			line_3 = song_data.album,
			line_4 = time_readable,
		})
	end
	
	if config.do_chat == "YES" then
		print("\127AAAAAA[Resound] " .. bstr(7, "Now Playing: ") .. song_data.name .. " from " .. song_data.album)
	end
end

engine:register_resound_update_callback(notif_handler)


if dbg then
	declare("rseclass", {})--create and update this table for debugging

	local updater = function()
		rseclass = engine
	end

	engine:new_scanning_tag(updater)
end

RegisterEvent(function()
	if config.adv_tags == "YES" then
		cp("Attempting to load additional environmental tags...")
		lib.resolve_file("plugins/Resound/extra_tags.lua")
	end
end, "LME_PLUGINS_LOADED")




update_class()

lib.require({{name="babel", version="0"}}, do_babel)